@@ -16,10 +16,10 @@
 """Identity v3 Role action implementations"""
 
 import logging
-import sys
 
 from keystoneauth1 import exceptions as ks_exc
 from osc_lib.command import command
+from osc_lib import exceptions
 from osc_lib import utils
 import six
 
@@ -31,13 +31,18 @@ LOG = logging.getLogger(__name__)
 
 
 def _add_identity_and_resource_options_to_parser(parser):
-    domain_or_project = parser.add_mutually_exclusive_group()
-    domain_or_project.add_argument(
+    system_or_domain_or_project = parser.add_mutually_exclusive_group()
+    system_or_domain_or_project.add_argument(
+        '--system',
+        metavar='<system>',
+        help=_('Include <system> (all)'),
+    )
+    system_or_domain_or_project.add_argument(
         '--domain',
         metavar='<domain>',
         help=_('Include <domain> (name or ID)'),
     )
-    domain_or_project.add_argument(
+    system_or_domain_or_project.add_argument(
         '--project',
         metavar='<project>',
         help=_('Include <project> (name or ID)'),
@@ -62,7 +67,14 @@ def _add_identity_and_resource_options_to_parser(parser):
 def _process_identity_and_resource_options(parsed_args,
                                            identity_client_manager):
     kwargs = {}
-    if parsed_args.user and parsed_args.domain:
+    if parsed_args.user and parsed_args.system:
+        kwargs['user'] = common.find_user(
+            identity_client_manager,
+            parsed_args.user,
+            parsed_args.user_domain,
+        ).id
+        kwargs['system'] = parsed_args.system
+    elif parsed_args.user and parsed_args.domain:
         kwargs['user'] = common.find_user(
             identity_client_manager,
             parsed_args.user,
@@ -83,6 +95,13 @@ def _process_identity_and_resource_options(parsed_args,
             parsed_args.project,
             parsed_args.project_domain,
         ).id
+    elif parsed_args.group and parsed_args.system:
+        kwargs['group'] = common.find_group(
+            identity_client_manager,
+            parsed_args.group,
+            parsed_args.group_domain,
+        ).id
+        kwargs['system'] = parsed_args.system
     elif parsed_args.group and parsed_args.domain:
         kwargs['group'] = common.find_group(
             identity_client_manager,
@@ -109,8 +128,8 @@ def _process_identity_and_resource_options(parsed_args,
 
 
 class AddRole(command.Command):
-    _description = _("Adds a role assignment to a user or group on a domain "
-                     "or project")
+    _description = _("Adds a role assignment to a user or group on the "
+                     "system, a domain, or a project")
 
     def get_parser(self, prog_name):
         parser = super(AddRole, self).get_parser(prog_name)
@@ -126,9 +145,11 @@ class AddRole(command.Command):
     def take_action(self, parsed_args):
         identity_client = self.app.client_manager.identity
 
-        if (not parsed_args.user and not parsed_args.domain
-                and not parsed_args.group and not parsed_args.project):
-            return
+        if (not parsed_args.user and not parsed_args.domain and
+                not parsed_args.group and not parsed_args.project):
+            msg = _("Role not added, incorrect set of arguments "
+                    "provided. See openstack --help for more details")
+            raise exceptions.CommandError(msg)
 
         domain_id = None
         if parsed_args.role_domain:
@@ -142,11 +163,6 @@ class AddRole(command.Command):
 
         kwargs = _process_identity_and_resource_options(
             parsed_args, self.app.client_manager.identity)
-        if not kwargs:
-            sys.stderr.write(_("Role not added, incorrect set of arguments "
-                               "provided. See openstack --help for more "
-                               "details\n"))
-            return
 
         identity_client.roles.grant(role.id, **kwargs)
 
@@ -223,14 +239,26 @@ class DeleteRole(command.Command):
         if parsed_args.domain:
             domain_id = common.find_domain(identity_client,
                                            parsed_args.domain).id
-
+        errors = 0
         for role in parsed_args.roles:
-            role_obj = utils.find_resource(
-                identity_client.roles,
-                role,
-                domain_id=domain_id
-            )
-            identity_client.roles.delete(role_obj.id)
+            try:
+                role_obj = utils.find_resource(
+                    identity_client.roles,
+                    role,
+                    domain_id=domain_id
+                )
+                identity_client.roles.delete(role_obj.id)
+            except Exception as e:
+                errors += 1
+                LOG.error(_("Failed to delete role with "
+                          "name or ID '%(role)s': %(e)s"),
+                          {'role': role, 'e': e})
+
+        if errors > 0:
+            total = len(parsed_args.roles)
+            msg = (_("%(errors)s of %(total)s roles failed "
+                   "to delete.") % {'errors': errors, 'total': total})
+            raise exceptions.CommandError(msg)
 
 
 class ListRole(command.Lister):
@@ -359,10 +387,10 @@ class ListRole(command.Lister):
                                '<group-name> --project <project-name> --names '
                                'instead.'))
         else:
-            sys.stderr.write(_("Error: If a user or group is specified, "
-                               "either --domain or --project must also be "
-                               "specified to list role grants.\n"))
-            return ([], [])
+            msg = _("Error: If a user or group is specified, "
+                    "either --domain or --project must also be "
+                    "specified to list role grants.")
+            raise exceptions.CommandError(msg)
 
         return (columns,
                 (utils.get_item_properties(
@@ -372,7 +400,7 @@ class ListRole(command.Lister):
 
 
 class RemoveRole(command.Command):
-    _description = _("Removes a role assignment from domain/project : "
+    _description = _("Removes a role assignment from system/domain/project : "
                      "user/group")
 
     def get_parser(self, prog_name):
@@ -390,11 +418,11 @@ class RemoveRole(command.Command):
     def take_action(self, parsed_args):
         identity_client = self.app.client_manager.identity
 
-        if (not parsed_args.user and not parsed_args.domain
-                and not parsed_args.group and not parsed_args.project):
-            sys.stderr.write(_("Incorrect set of arguments provided. "
-                               "See openstack --help for more details\n"))
-            return
+        if (not parsed_args.user and not parsed_args.domain and
+                not parsed_args.group and not parsed_args.project):
+            msg = _("Incorrect set of arguments provided. "
+                    "See openstack --help for more details")
+            raise exceptions.CommandError(msg)
 
         domain_id = None
         if parsed_args.role_domain:
@@ -408,11 +436,6 @@ class RemoveRole(command.Command):
 
         kwargs = _process_identity_and_resource_options(
             parsed_args, self.app.client_manager.identity)
-        if not kwargs:
-            sys.stderr.write(_("Role not removed, incorrect set of arguments "
-                               "provided. See openstack --help for more "
-                               "details\n"))
-            return
         identity_client.roles.revoke(role.id, **kwargs)
 
 
